#include "unInstall.h"

#include <stdio.h>
#include <stdarg.h>
#include "File.h"

bool installManager::openLog(string filename, bool write)
{ string command;
	stringArray params;

	if (this->logfile.rdbuf()->is_open())
	{	this->closeLog();
  }
  this->logfileName = filename;
  this->logfile.open(filename.c_str(), ios::out | ios::in);
  if (write == false)
  { while ((command = this->readCommand(params)) != "")
  	{ if (command[0] == '[' && command[command.length()-1] == ']')
  		{ this->setModule(command.substr(1, command.length() - 2));
    	}
    	else
    	{ unInstallCommand action(command, params);
    		this->storeCommand(action);
    	}
    }
  }
  this->setModule("default");
  return true;
}

bool installManager::reopenLog()
{ if (this->logfile.rdbuf()->is_open())
  { this->logfile.close();
  }
  else
  { this->logfile.close();
	}
  
  this->logfile.open(this->logfileName.c_str(), ios::out | ios::in);
  return true;
}

void installManager::setModule(string moduleName)
{	this->currentModule = moduleName;
}

bool installManager::storeCommand(unInstallCommand &command)
{ this->modules[this->currentModule].push_back(command);
	return true;
}

bool installManager::writeCommand(unInstallCommand &command)
{ if (!this->writeString(command.command))
	{ return false;
  }
	for (unsigned int p = 0; p < command.parameters.size(); p ++)
  {	if (!this->writeSeparator())
  	{ return false;
    }
    if (!this->writeString(command.parameters[p]))
    { return false;
    }
  }
  if (!this->writeString("\n"))
  { return false;
  }
  return true;
}

bool installManager::writeString(char *buffer)
{ string s(buffer);
	return this->writeString(s);
}

bool installManager::writeString(string str)
{ bool quote;

	// TODO: check for space characters in str and quote if necessary
  quote = str.find_first_of(' ') < str.length();

	if (quote)
  {	this->logfile << "\"";
  }
	this->logfile << str;
  if (quote)
  {	this->logfile << "\"";
  }
  return true;
}

bool installManager::writeSeparator()
{	this->logfile << " ";
	return true;
}

string installManager::readString()
{	string reply = "";
	char c;

  if (this->logfile.eof())
	{	return reply;
  }

  this->logfile >> c;
  while (c <= ' ' && !this->logfile.eof())
  {	this->logfile.get();
  }

  if (this->logfile.eof())
  { return reply;
  }

  if (c == '\"')
  { do
  	{	c = this->logfile.get();
    	if (c != '\"')
      { reply += c;
      }
    }
    while (c != '\"' && !this->logfile.eof());
  }
  else
  { while (c > ' ')
    { reply += c;
    	if (this->logfile.eof())
      {	break;
      }
    	c = this->logfile.get();
    }
    if (!this->logfile.eof())
    {	this->logfile.putback(c);
    }
    //this->logfile >> reply;
  }
  return reply;
}

string installManager::readCommand(stringArray &array)
{	string reply = "";
  char c;

  array.clear();

  if (this->logfile.eof())
  {	return reply;
  }

	reply = this->readString();
  if (reply == "")
  { return reply;
  }

  while(!this->logfile.eof() &&
  			(c = this->logfile.get()) != '\n')
  {	this->logfile.putback(c);
   	array.add(this->readString());
  }
  if (!this->logfile.eof())
  { this->logfile.putback(c);
  }

  return reply;
}

string installManager::popCommand(stringArray &array)
{ string command;

  if (this->modules[this->currentModule].size() == 0)
  { command = "";
  }
  else
  { unsigned int last;

	  last = this->modules[this->currentModule].size() - 1;
  	command = this->modules[this->currentModule][last].commandName();
  	array		= this->modules[this->currentModule][last].parameterList();
    this->modules[this->currentModule].erase(this->modules[this->currentModule].begin() + last);
  }
  return command;
}

bool installManager::closeLog()
{ if (this->logfile.rdbuf()->is_open() == false)
  { return false;
  }

	unInstallCommandMap::iterator here = this->modules.begin();
	unInstallCommandMap::iterator end  = this->modules.end();

  while (here != end)
  {	this->writeString("[" + (*here).first + "]\n");

  	unInstallCommandList::iterator	ahere = (*here).second.begin();
    unInstallCommandList::iterator  aend  = (*here).second.end();
    while (ahere != aend)
		{ this->writeCommand(*ahere);
    	ahere ++;
    }
  	here ++;
  }
	this->logfile.close();
  return true;
}

bool installManager::isEmpty()
{ unInstallCommandMap::iterator here = this->modules.begin();
	unInstallCommandMap::iterator end  = this->modules.end();

  while (here != end)
  {	this->writeString("[" + (*here).first + "]\n");

  	unInstallCommandList::iterator	ahere = (*here).second.begin();
    unInstallCommandList::iterator  aend  = (*here).second.end();
    if (ahere != aend)
    {	return false;
    }
  	here ++;
  }
  return true;
}

installManager::~installManager()
{	if (this->logfile.rdbuf()->is_open())
	{	this->closeLog();
  }
}


